@@ -14,7 +14,6 @@
   +----------------------------------------------------------------------+
 */
 
-
 #ifndef YAF_ROUTER_SUPERVAR_H
 #define YAF_ROUTER_SUPERVAR_H
 
@@ -22,7 +21,7 @@
 
 extern zend_class_entry *yaf_route_supervar_ce;
 
-yaf_route_t * yaf_route_supervar_instance(yaf_route_t *this_ptr, zval *name TSRMLS_DC);
+yaf_route_t *yaf_route_supervar_instance(yaf_route_t *this_ptr, zval *name);
 
 YAF_STARTUP_FUNCTION(route_supervar);
 
