@@ -149,11 +149,6 @@ PHP_MSHUTDOWN_FUNCTION(yaf)
 {
 	UNREGISTER_INI_ENTRIES();
 
-	if (YAF_G(configs)) {
-		zend_hash_destroy(YAF_G(configs));
-		pefree(YAF_G(configs), 1);
-	}
-
 	return SUCCESS;
 }
 /* }}} */
@@ -193,7 +188,7 @@ PHP_RSHUTDOWN_FUNCTION(yaf)
 	}
 	if (YAF_G(local_namespaces)) {
 #if PHP_VERSION_ID >70300
-		if (zend_gc_delref(YAF_G(local_namespaces)) == 0)
+		if (GC_DELREF(YAF_G(local_namespaces)) == 0)
 #else
 		if (--(GC_REFCOUNT(YAF_G(local_namespaces))) == 0)
 #endif
@@ -208,7 +203,7 @@ PHP_RSHUTDOWN_FUNCTION(yaf)
 	}
 	if (YAF_G(modules)) {
 #if PHP_VERSION_ID >70300
-		if (zend_gc_delref(YAF_G(modules)) == 0)
+		if (GC_DELREF(YAF_G(modules)) == 0)
 #else
 		if (--(GC_REFCOUNT(YAF_G(modules))) == 0)
 #endif
