@@ -123,7 +123,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				if (script_name && EXPECTED(IS_STRING == Z_TYPE_P(script_name))) {
 					zend_string	*script = php_basename(Z_STRVAL_P(script_name), Z_STRLEN_P(script_name), NULL, 0);
 
-					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(script), ZSTR_LEN(file_name)) == 0) {
+					if (memcmp(ZSTR_VAL(file_name), ZSTR_VAL(script), MIN(ZSTR_LEN(file_name), ZSTR_LEN(script))) == 0) {
 						basename = zend_string_copy(Z_STR_P(script_name));
 						zend_string_release(file_name);
 						zend_string_release(script);
@@ -135,7 +135,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				phpself_name = yaf_request_query_str(YAF_GLOBAL_VARS_SERVER, "PHP_SELF", sizeof("PHP_SELF") - 1);
 				if (phpself_name && EXPECTED(IS_STRING == Z_TYPE_P(phpself_name))) {
 					zend_string *phpself = php_basename(Z_STRVAL_P(phpself_name), Z_STRLEN_P(phpself_name), NULL, 0);
-					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(phpself), ZSTR_LEN(file_name)) == 0) {
+					if (memcmp(ZSTR_VAL(file_name), ZSTR_VAL(phpself), MIN(ZSTR_LEN(file_name), ZSTR_LEN(file_name))) == 0) {
 						basename = zend_string_copy(Z_STR_P(phpself_name));
 						zend_string_release(file_name);
 						zend_string_release(phpself);
@@ -147,7 +147,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 				orig_name = yaf_request_query_str(YAF_GLOBAL_VARS_SERVER, "ORIG_SCRIPT_NAME", sizeof("ORIG_SCRIPT_NAME") - 1);
 				if (orig_name && IS_STRING == Z_TYPE_P(orig_name)) {
 					zend_string *orig = php_basename(Z_STRVAL_P(orig_name), Z_STRLEN_P(orig_name), NULL, 0);
-					if (memcpy(ZSTR_VAL(file_name), ZSTR_VAL(orig), ZSTR_LEN(file_name)) == 0) {
+					if (memcmp(ZSTR_VAL(file_name), ZSTR_VAL(orig), MIN(ZSTR_LEN(file_name), ZSTR_LEN(file_name))) == 0) {
 						basename = zend_string_copy(Z_STR_P(orig_name));
 						zend_string_release(file_name);
 						zend_string_release(orig);
@@ -160,13 +160,14 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 		} while (0);
 
 		if (basename) {
-			if (memcpy(ZSTR_VAL(request_uri), ZSTR_VAL(basename), ZSTR_LEN(basename)) == 0) {
+			if (memcmp(ZSTR_VAL(request_uri), ZSTR_VAL(basename), MIN(ZSTR_LEN(basename), ZSTR_LEN(request_uri))) == 0) {
 				if (ZSTR_VAL(basename)[ZSTR_LEN(basename) - 1] == '/') {
 					zend_string *sanitized_uri = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename) - 1, 0);
 					zend_string_release(basename);
 					basename = sanitized_uri;
 				}
 				zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), basename);
+				zend_string_release(basename);
 				return 1;
 			} else {
 				zend_string *dir = zend_string_init(ZSTR_VAL(basename), ZSTR_LEN(basename), 0); /* php_dirname might alter the string */
@@ -178,7 +179,7 @@ int yaf_request_set_base_uri(yaf_request_t *request, zend_string *base_uri, zend
 					ZSTR_LEN(dir)--;
 				}
 				if (ZSTR_LEN(dir)) {
-					if (memcpy(ZSTR_VAL(request_uri), ZSTR_VAL(dir), ZSTR_LEN(dir)) == 0) {
+					if (memcmp(ZSTR_VAL(request_uri), ZSTR_VAL(dir), MIN(ZSTR_LEN(dir), ZSTR_LEN(request_uri))) == 0) {
 						zend_update_property_str(yaf_request_ce, request, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_BASE), dir);
 						zend_string_release(dir);
 						return 1;
@@ -243,6 +244,10 @@ zval *yaf_request_query_str(unsigned type, const char *name, size_t len) /* {{{
 		return NULL;
 	}
 
+	if (name == NULL) {
+		return container;
+	}
+
 	return zend_hash_str_find(Z_ARRVAL_P(container), name, len);
 }
 /* }}} */
@@ -254,6 +259,10 @@ zval *yaf_request_query(unsigned type, zend_string *name) /* {{{ */ {
 		return NULL;
 	}
 
+	if (UNEXPECTED(!name)) {
+		return container;
+	}
+
 	return zend_hash_find(Z_ARRVAL_P(container), name);
 }
 /* }}} */
@@ -292,17 +301,14 @@ zval *yaf_request_get_language(yaf_request_t *instance, zval *accept_language) /
 					float qval = strtod(qvalue + 2, NULL);
 					if (qval > max_qvlaue) {
 						max_qvlaue = qval;
-						if (prefer) {
-							efree(prefer);
-						}
 						prefer_len = qvalue - seg - 1;
-						prefer 	   = estrndup(seg, prefer_len);
+						prefer 	   = seg;
 					}
 				} else {
 					if (max_qvlaue < 1) {
 						max_qvlaue = 1;
 						prefer_len = strlen(seg);
-						prefer 	   = estrndup(seg, prefer_len);
+						prefer 	   = seg;
 					}
 				}
 
@@ -310,10 +316,9 @@ zval *yaf_request_get_language(yaf_request_t *instance, zval *accept_language) /
 			}
 
 			if (prefer) {
-				ZVAL_STRINGL(accept_language, prefer, prefer_len);
+				ZVAL_NEW_STR(accept_language, zend_string_init(prefer, prefer_len, 1));
 				zend_update_property(yaf_request_ce,
 						instance, ZEND_STRL(YAF_REQUEST_PROPERTY_NAME_LANG), accept_language);
-				efree(prefer);
 				efree(langs);
 				zval_ptr_dtor(accept_language);
 				return accept_language;
